"""
Author:         David Szybala
Website:        www.davidszybala.com
Alt Website:    www.twistedgerbil.com
Program:        bitmap_cleaner.py
Date:           11/07/14
Description:    This scripts provides a number of methods for altering
                bitmap paths.
Installation:   Place this file in your maya scripts directory
                (e.g. ..\My Documents\maya\scripts) then run the following
                python commands:
                    import bitmap_cleaner
                    bitmap_cleaner.main()
"""
import maya.cmds as cmds
import maya.mel as mel
import os
import re


class BitmapCleaner(object):
    def __init__(self):
        self.bitmap_nodes = []
        self.bitmap_paths = []
        self.search_text_group = None
        self.include_sub_directory_checkbox = None
        self.mode_path_radio = None
        self.mode_filename_radio = None
        self.mode_extension_radio = None
        self.mode_all_radio = None
        self.modify_text = None
        self.find_text = None
        self.replace_text = None
        self.match_case_checkbox = None
        self.first_only_checkbox = None
        self.bitmap_list_text = None
        self.bitmap_list = None
        self.status_label = None

    @staticmethod
    def is_hypershade_open():
        """Returns True if the hypershade panel is open"""
        all_windows = cmds.lsUI(windows=True)

        if 'hyperShadePanel1Window' in all_windows:
            return True
        else:
            return False

    @staticmethod
    def get_bitmap_nodes():
        """Returns a list of bitmap nodes"""
        bitmap_nodes = cmds.ls(type='file', long=True)
        bitmap_nodes.sort()

        return bitmap_nodes

    @staticmethod
    def get_bitmap_paths(node_list):
        """Returns a unique sorted list of bitmap paths from 'node_list' """
        bitmap_paths = [cmds.getAttr(x + '.fileTextureName') for x in node_list]
        bitmap_paths = set(bitmap_paths)
        bitmap_paths = list(bitmap_paths)
        bitmap_paths.sort()

        return bitmap_paths

    @staticmethod
    def search_for_file(directory, file_path, include_subs):
        """Searches 'search_directory' for the file name from 'file_path'
        Returns the full file path if found"""
        file_name = os.path.basename(file_path)
        full_path = ''

        if include_subs:
            for root, dirs, files in os.walk(directory):
                for current in files:
                    if current == file_name:
                        full_path = os.path.join(root, current)
                        full_path = os.path.normpath(full_path)
                        return full_path
        else:
            if file_name in os.listdir(directory):
                full_path = os.path.join(directory, file_name)
                full_path = os.path.normpath(full_path)

        return full_path

    def set_bitmap_paths(self, from_path, to_path):
        """Changes all bitmap nodes using 'from_path' to 'to_path' """
        cmds.scriptEditorInfo(suppressWarnings=True, suppressInfo=True,
                              suppressResults=True)

        from_path = from_path.replace('\\', '/')

        for current in self.bitmap_nodes:
            current_path = cmds.getAttr(current + '.fileTextureName')
            if current_path.replace('\\', '/') == from_path:
                cmds.setAttr(current + '.fileTextureName', to_path, type='string')

        cmds.scriptEditorInfo(suppressWarnings=False, suppressInfo=False,
                              suppressResults=False)

    @staticmethod
    def get_modify_path(file_path, modify_text, edit_mode):
        """Returns the 'file_path' after editing it with 'modify_text' """
        if edit_mode == 1:
            file_path = os.path.join(modify_text, os.path.basename(file_path))
        elif edit_mode == 2:
            file_path = os.path.join(os.path.dirname(file_path),
                                     modify_text + os.path.splitext(file_path)[-1])
        elif edit_mode == 3:
            while modify_text.startswith('.'):
                modify_text = modify_text[1:]
            file_path = os.path.splitext(file_path)[0] + '.' + modify_text
        else:
            file_path = modify_text

        file_path = os.path.normpath(file_path)

        return file_path

    @staticmethod
    def get_find_replace(name, find_text, replace_text, match_case, first_only):
        """Performs a find and replace and returns the result"""
        count = 0

        if first_only:
            count = 1

        if find_text:
            if match_case:
                if count == 0:
                    # .replace takes -1 for 'all occurrences'
                    count = -1
                name = name.replace(find_text, replace_text, count)
            else:
                # CRITICAL! You must escape the special characters
                special_chars = '.^$*+?{}[]|()'
                find_text = find_text.replace('\\', '\\\\')
                for char in special_chars:
                    find_text = find_text.replace(char, '\\' + char)

                name = re.sub(find_text, replace_text, name, count, re.IGNORECASE)

        return name

    def get_find_replace_path(self, file_path, find_text, replace_text,
                              match_case, first_only, edit_mode):
        """Returns the 'file_path' after performing a find and replace """
        if edit_mode == 1:
            result = self.get_find_replace(os.path.dirname(file_path),
                                           find_text, replace_text,
                                           match_case, first_only)
            file_path = os.path.join(result, os.path.basename(file_path))
        elif edit_mode == 2:
            file_name = os.path.basename(file_path)
            file_name = os.path.splitext(file_name)[0]
            result = self.get_find_replace(file_name,
                                           find_text, replace_text,
                                           match_case, first_only)
            file_path = os.path.join(os.path.dirname(file_path),
                                     result + os.path.splitext(file_path)[-1])
        elif edit_mode == 3:
            result = self.get_find_replace(os.path.splitext(file_path)[-1],
                                           find_text, replace_text,
                                           match_case, first_only)
            while result.startswith('.'):
                result = result[1:]
            file_path = os.path.splitext(file_path)[0] + '.' + result
        else:
            file_path = self.get_find_replace(file_path,
                                              find_text, replace_text,
                                              match_case, first_only)

        file_path = os.path.normpath(file_path)

        return file_path

    def get_nodes_using_path(self, file_path):
        """Returns a list of bitmap nodes that are referencing path
        'file_path' """
        nodes = []

        file_path = file_path.replace('\\', '/')

        for current in self.bitmap_nodes:
            current_path = cmds.getAttr(current + '.fileTextureName')
            if current_path.replace('\\', '/') == file_path:
                nodes.append(current)

        return nodes

    @staticmethod
    def get_absolute_path(file_path):
        """Returns the absolute path of 'file_path' """
        full_path = ''

        if file_path:
            full_path = cmds.file(file_path, query=True, expandName=True)

        return full_path

    def get_missing_maps(self):
        """Returns a dictionary of missing maps. Format = <index>:<map>"""
        missing_maps = {}

        for i in range(0, len(self.bitmap_paths)):
            if not os.path.exists(self.get_absolute_path(self.bitmap_paths[i])):
                missing_maps[i] = self.bitmap_paths[i]
                continue

        return missing_maps

    def refresh_bitmap_list(self, *args):
        """Refreshes the bitmap list"""
        self.bitmap_nodes = self.get_bitmap_nodes()
        self.bitmap_paths = self.get_bitmap_paths(self.bitmap_nodes)
        cmds.text(self.bitmap_list_text, edit=True,
                  label=('Maps: ' + str(len(self.bitmap_paths)) + ' Nodes: ' + str(len(self.bitmap_nodes))))
        cmds.textScrollList(self.bitmap_list, edit=True, removeAll=True)
        cmds.textScrollList(self.bitmap_list, edit=True, append=self.bitmap_paths)
        cmds.text(self.status_label, edit=True, label='INFO: Bitmap list updated')

    def search_maps_browse_button_pressed(self, *args):
        """Search maps browse button event handler. Opens a dialog allowing
        the user to select a directory"""
        current_path = cmds.textFieldButtonGrp(self.search_text_group, query=True, text=True)
        new_path = cmds.fileDialog2(dialogStyle=2, caption='Select a Search Directory',
                                    fileMode=3, startingDirectory=current_path)
        if new_path:
            cmds.textFieldButtonGrp(self.search_text_group, edit=True, text=new_path[0])
            cmds.text(self.status_label, edit=True, label='INFO: Search directory updated')
        else:
            cmds.text(self.status_label, edit=True,
                      label='WARNING: Directory selection cancelled by user')

    def search_maps_button_pressed(self, *args):
        """Search maps button event handler. Performs a search for the selected
        bitmap paths"""
        user_selection = cmds.textScrollList(self.bitmap_list, query=True, selectItem=True)
        search_directory = cmds.textFieldButtonGrp(self.search_text_group, query=True, text=True)
        include_subs = cmds.checkBox(self.include_sub_directory_checkbox, query=True, value=True)
        counter = 0
        error_counter = 0

        if not user_selection:
            cmds.text(self.status_label, edit=True, label='ERROR: No paths selected')
            return False

        if not search_directory:
            cmds.text(self.status_label, edit=True, label='ERROR: Search directory is blank')
            return False

        if not os.path.exists(search_directory):
            cmds.text(self.status_label, edit=True, label='ERROR: Search directory does not exist')
            return False

        cmds.text(self.status_label, edit=True, label='INFO: Searching for maps. Please wait...')
        cmds.refresh(currentView=True)

        for current in user_selection:
            new_path = self.search_for_file(search_directory, current, include_subs)

            if new_path:
                self.set_bitmap_paths(current, new_path)
                counter += 1
            else:
                print('ERROR: Could not find file ' + current)
                error_counter += 1

        if counter > 0:
            self.refresh_bitmap_list()

        if error_counter == 0:
            message = 'INFO: Search complete. '
            message += str(counter) + ' maps found'
        else:
            message = 'WARNING: Search complete. '
            message += str(error_counter) + ' maps could not be found. '
            message += 'Check script editor'
        cmds.text(self.status_label, edit=True, label=message)

    def get_edit_mode(self):
        """Returns the edit mode as an integer"""
        mode = 4

        if cmds.radioButton(self.mode_path_radio, query=True, select=True):
            mode = 1
        elif cmds.radioButton(self.mode_filename_radio, query=True, select=True):
            mode = 2
        elif cmds.radioButton(self.mode_extension_radio, query=True, select=True):
            mode = 3

        return mode

    def modify_button_pressed(self, *args):
        """Modify button event handler. Performs a replacement on the selected
        bitmap paths"""
        user_selection = cmds.textScrollList(self.bitmap_list, query=True, selectItem=True)
        modify_text = cmds.textField(self.modify_text, query=True, text=True)
        edit_mode = self.get_edit_mode()

        if not user_selection:
            cmds.text(self.status_label, edit=True, label='ERROR: No paths selected')
            return False

        if not modify_text:
            cmds.text(self.status_label, edit=True, label='ERROR: Modify text is blank')
            return False

        cmds.text(self.status_label, edit=True, label='INFO: Modifying paths. Please wait...')
        cmds.refresh(currentView=True)

        for current in user_selection:
            new_path = self.get_modify_path(current, modify_text, edit_mode)
            self.set_bitmap_paths(current, new_path)

        self.refresh_bitmap_list()
        cmds.text(self.status_label, edit=True, label='INFO: Modify paths complete')

    def find_replace_button_pressed(self, *args):
        """Find/Replace button event handler. Performs a find and replace
        operation on the selected bitmap paths"""
        user_selection = cmds.textScrollList(self.bitmap_list, query=True, selectItem=True)
        find_text = cmds.textFieldGrp(self.find_text, query=True, text=True)
        replace_text = cmds.textFieldGrp(self.replace_text, query=True, text=True)
        match_case = cmds.checkBox(self.match_case_checkbox, query=True, value=True)
        first_only = cmds.checkBox(self.first_only_checkbox, query=True, value=True)
        edit_mode = self.get_edit_mode()

        if not user_selection:
            cmds.text(self.status_label, edit=True, label='ERROR: No paths selected')
            return False

        if not find_text:
            cmds.text(self.status_label, edit=True, label='ERROR: Find text is blank')
            return False

        cmds.text(self.status_label, edit=True, label='INFO: Find/Replacing paths. Please wait...')
        cmds.refresh(currentView=True)

        for current in user_selection:
            new_path = self.get_find_replace_path(current, find_text, replace_text,
                                                  match_case, first_only, edit_mode)
            self.set_bitmap_paths(current, new_path)

        self.refresh_bitmap_list()
        cmds.text(self.status_label, edit=True, label='INFO: Find/Replace paths complete')

    def select_missing_maps_button_pressed(self, *args):
        """Select missing maps event handler. Select all maps in the bitmap
        list that do not exist"""
        message = 'INFO: '
        missing_maps = self.get_missing_maps()
        cmds.textScrollList(self.bitmap_list, edit=True, deselectAll=True)

        if missing_maps:
            cmds.textScrollList(self.bitmap_list, edit=True,
                                selectIndexedItem=[x + 1 for x in missing_maps.keys()])
            message = 'WARNING: '

        message += str(len(missing_maps.keys())) + ' missing maps found'
        cmds.text(self.status_label, edit=True, label=message)

    def export_bitmaps(self, update_paths):
        """Exports selected bitmaps to the user defined location. If
        'update_paths' the paths in the current scene are changed to the export
        directory"""
        user_selection = cmds.textScrollList(self.bitmap_list, query=True, selectItem=True)
        error_counter = 0

        if not user_selection:
            cmds.text(self.status_label, edit=True, label='ERROR: No paths selected')
            return False

        export_directory = cmds.fileDialog2(dialogStyle=2, caption="Export Location",
                                            fileMode=3)
        if not export_directory:
            cmds.text(self.status_label, edit=True, label='WARNING: Export bitmaps cancelled by user')
            return False

        cmds.text(self.status_label, edit=True, label='INFO: Exporting maps. Please wait...')
        cmds.refresh(currentView=True)

        for current in user_selection:
            source_path = self.get_absolute_path(current)

            if not os.path.exists(source_path):
                print('ERROR: Could not find file ' + source_path)
                error_counter += 1
                continue

            dest_path = os.path.join(export_directory[0],
                                     os.path.basename(source_path))
            dest_path = os.path.normpath(dest_path)

            if os.path.exists(dest_path):
                if not cmds.sysFile(dest_path, delete=True):
                    print('WARNING: Could not update existing file ' + dest_path)
                    error_counter += 1
                    continue

            if cmds.sysFile(source_path, copy=dest_path):
                if update_paths:
                    self.set_bitmap_paths(current, dest_path)
            else:
                print('ERROR: Could not copy file ' + current)
                error_counter += 1

        if update_paths:
            self.refresh_bitmap_list()

        if error_counter == 0:
            message = 'INFO: Export complete. '
            message += str(len(user_selection)) + ' maps exported'
        else:
            message = 'WARNING: Export complete. '
            message += str(error_counter) + ' maps could not be found. '
            message += 'Check script editor'
        cmds.text(self.status_label, edit=True, label=message)

    def export_bitmaps_button_pressed(self, *args):
        """Export bitmaps event handler. Exports the selected bitmaps"""
        self.export_bitmaps(False)

    def export_bitmaps_update_button_pressed(self, *args):
        """Export bitmaps and update event handler. Exports the selected
        bitmaps and updates the scene paths"""
        self.export_bitmaps(True)

    def graph_bitmap_button_pressed(self, *args):
        """Graphs the selected bitmaps in the hypershade"""
        user_selection = cmds.textScrollList(self.bitmap_list, query=True, selectItem=True)

        if not user_selection:
            cmds.text(self.status_label, edit=True, label='ERROR: No paths selected')
            return False

        if not self.is_hypershade_open():
            cmds.HypershadeWindow()

        nodes = []
        cmds.select(clear=True)
        for current in user_selection:
            nodes.extend(self.get_nodes_using_path(current))
        cmds.select(nodes)

        mel.eval('hyperShadePanelGraphCommand("hyperShadePanel1", "showUpAndDownstream");')
        cmds.text(self.status_label, edit=True, label='INFO: Bitmap nodes have been graphed')

    def view_bitmap_button_pressed(self, *args):
        """Displays the selected bitmaps using fcheck"""
        user_selection = cmds.textScrollList(self.bitmap_list, query=True, selectItem=True)
        error_counter = 0

        if not user_selection:
            cmds.text(self.status_label, edit=True, label='ERROR: No paths selected')
            return False

        cmds.text(self.status_label, edit=True, label='INFO: Displaying maps. Please wait...')
        cmds.refresh(currentView=True)

        for current in user_selection:
            if os.path.exists(self.get_absolute_path(current)):
                cmds.fcheck(current)
            else:
                print('ERROR: Could not display file ' + current)
                error_counter += 1

        if error_counter == 0:
            message = 'INFO: Displaying ' + str(len(user_selection)) + ' maps'
        else:
            message = 'WARNING: Could not display ' + str(error_counter) + ' maps. '
            message += 'Check script editor'
        cmds.text(self.status_label, edit=True, label=message)

    def view_directory_button_pressed(self, *args):
        """Displays the selected bitmaps directories"""
        user_selection = cmds.textScrollList(self.bitmap_list, query=True, selectItem=True)
        error_counter = 0

        if not user_selection:
            cmds.text(self.status_label, edit=True, label='ERROR: No paths selected')
            return False

        for current in user_selection:
            full_path = self.get_absolute_path(current)

            if current:
                cmds.launch(directory=os.path.dirname(full_path))
            else:
                print('ERROR: Could not view bitmap directory ' + full_path)
                error_counter += 1

        if error_counter == 0:
            message = 'INFO: Displaying ' + str(len(user_selection)) + ' directories'
        else:
            message = 'WARNING: Could not display ' + str(error_counter) + ' directories. '
            message += 'Check script editor'
        cmds.text(self.status_label, edit=True, label=message)

    def delete_bitmap_button_pressed(self, *args):
        """Deletes the selected bitmap nodes"""
        user_selection = cmds.textScrollList(self.bitmap_list, query=True, selectItem=True)
        delete_nodes = []
        error_counter = 0

        if not user_selection:
            cmds.text(self.status_label, edit=True, label='ERROR: No paths selected')
            return False

        delete_ok = cmds.confirmDialog(title="DELETE CONFIRMATION",
                                       message="Are you sure you want to delete these file nodes?",
                                       button=["Yes", "No"], defaultButton="No",
                                       cancelButton="No", dismissString="No",
                                       icon="warning")

        if delete_ok != 'Yes':
            cmds.text(self.status_label, edit=True, label='WARNING: Delete nodes cancelled by user')
            return False

        for current in user_selection:
            delete_nodes.extend(self.get_nodes_using_path(current))

        cmds.delete(delete_nodes)

        for current in delete_nodes:
            if cmds.objExists(current):
                print("ERROR: Could not delete node " + current)
                error_counter += 1

        self.refresh_bitmap_list()

        if error_counter == 0:
            message = 'INFO: Deleted ' + str(len(delete_nodes)) + ' nodes'
        else:
            message = 'WARNING: Could not delete ' + str(error_counter) + ' nodes. '
            message += 'Check script editor'
        cmds.text(self.status_label, edit=True, label=message)

    def select_all_button_pressed(self, *args):
        """Select all button event handler. Selects all the bitmap paths"""
        cmds.textScrollList(self.bitmap_list, edit=True,
                            selectIndexedItem=[(i + 1) for i in range(0, len(self.bitmap_paths))])
        cmds.text(self.status_label, edit=True, label='INFO: All paths have been selected')

    def select_invert_button_pressed(self, *args):
        """Select invert button event handler. Toggles the bitmap path
        selection"""
        current_selection = cmds.textScrollList(self.bitmap_list, query=True, selectIndexedItem=True)
        if not current_selection:
            current_selection = []

        selection_invert = [(i + 1) for i in range(0, len(self.bitmap_paths)) if (i + 1) not in current_selection]
        cmds.textScrollList(self.bitmap_list, edit=True, deselectAll=True)
        cmds.textScrollList(self.bitmap_list, edit=True, selectIndexedItem=selection_invert)
        cmds.text(self.status_label, edit=True, label='INFO: Path selection has been inverted')

    def select_none_button_pressed(self, *args):
        """Select none button event handler. Deselects all the bitmap paths"""
        cmds.textScrollList(self.bitmap_list, edit=True, deselectAll=True)
        cmds.text(self.status_label, edit=True, label='INFO: All paths have been deselected')

    def setup_ui(self):
        """Create the UI"""
        if cmds.window("bitmapCleanerWindow", exists=True):
            cmds.deleteUI("bitmapCleanerWindow")

        bitmap_window = cmds.window("bitmapCleanerWindow", title="Bitmap Cleaner",
                                    width=800, height=400,
                                    minimizeButton=True, maximizeButton=True,
                                    sizeable=True, resizeToFitChildren=True)
        main_layout = cmds.formLayout(numberOfDivisions=10, width=500)

        # Tab control
        tab_layout = cmds.tabLayout(width=300)

        # Search tab
        search_tab = cmds.columnLayout(width=300, columnAttach=('both', 5), rowSpacing=5)
        cmds.frameLayout(width=280, label='Directory',
                         collapsable=False, marginHeight=5, marginWidth=5)
        cmds.columnLayout(width=270)
        self.search_text_group = cmds.textFieldButtonGrp(width=270, label='', buttonLabel=' ... ',
                                                         columnWidth3=(0, 240, 30),
                                                         buttonCommand=self.search_maps_browse_button_pressed)
        self.include_sub_directory_checkbox = cmds.checkBox(label='Include Sub Directories', value=True)
        cmds.text(label='', height=10)
        cmds.button(width=280, label='Search for Selected Maps',
                    command=self.search_maps_button_pressed)
        cmds.setParent('..')  # end columnLayout
        cmds.setParent('..')  # end frameLayout
        cmds.setParent('..')  # end columnLayout

        # Edit tab
        edit_tab = cmds.columnLayout(width=300, columnAttach=('both', 5), rowSpacing=5)
        cmds.frameLayout(width=280, label='Mode',
                         collapsable=False, marginHeight=5, marginWidth=5)
        cmds.rowLayout(width=270, numberOfColumns=4)
        cmds.radioCollection()
        self.mode_path_radio = cmds.radioButton(label='Path', select=True)
        self.mode_filename_radio = cmds.radioButton(label='Filename')
        self.mode_extension_radio = cmds.radioButton(label='Extension')
        self.mode_all_radio = cmds.radioButton(label='All')
        cmds.setParent('..')  # end rowLayout
        cmds.setParent('..')  # end frameLayout
        cmds.frameLayout(width=280, label='Modify',
                         collapsable=False, marginHeight=5, marginWidth=5)
        cmds.columnLayout(width=270, rowSpacing=5)
        self.modify_text = cmds.textField(width=270)
        cmds.button(width=270, label='Modify Selected Maps',
                    command=self.modify_button_pressed)
        cmds.setParent('..')  # end columnLayout
        cmds.setParent('..')  # end frameLayout
        cmds.frameLayout(width=280, label='Find and Replace',
                         collapsable=False, marginHeight=5, marginWidth=5)
        cmds.columnLayout(width=270)
        self.find_text = cmds.textFieldGrp(label='Find:',
                                           columnAlign=[1, 'left'],
                                           columnWidth2=[50, 220])
        self.replace_text = cmds.textFieldGrp(label='Replace:',
                                              columnAlign=[1, 'left'],
                                              columnWidth2=[50, 220])
        self.match_case_checkbox = cmds.checkBox(label='Match Case',
                                                 value=True)
        self.first_only_checkbox = cmds.checkBox(label='First Occurence Only')
        cmds.text(label='', height=5)
        cmds.button(width=270, label='Find/Replace Selected Maps',
                    command=self.find_replace_button_pressed)
        cmds.setParent('..')  # end columnLayout
        cmds.setParent('..')  # end frameLayout
        cmds.setParent('..')  # end columnLayout

        # Misc tab
        misc_tab = cmds.columnLayout(width=300, columnAttach=('both', 5), rowSpacing=5)
        cmds.text(label='', height=5)
        cmds.button(width=280, label='Select Missing Maps',
                    command=self.select_missing_maps_button_pressed)
        cmds.button(width=280, label='Export Selected Maps',
                    command=self.export_bitmaps_button_pressed)
        cmds.button(width=280, label='Export Selected Maps and Update Scene Paths',
                    command=self.export_bitmaps_update_button_pressed)
        cmds.button(width=280, label='Graph Selected Maps',
                    command=self.graph_bitmap_button_pressed)
        cmds.button(width=280, label='View Selected Maps',
                    command=self.view_bitmap_button_pressed)
        cmds.button(width=280, label='View Selected Directories',
                    command=self.view_directory_button_pressed)
        cmds.button(width=280, label='Delete File Nodes Using Selected Maps',
                    command=self.delete_bitmap_button_pressed)
        cmds.setParent('..')

        cmds.tabLayout(tab_layout, edit=True,
                       tabLabel=[(search_tab, 'Search'), (edit_tab, 'Edit'), (misc_tab, 'Misc')])
        cmds.setParent('..')  # end tabLayout

        self.bitmap_list_text = cmds.text(label='Maps: 5 Nodes: 400', align='left')
        self.bitmap_list = cmds.textScrollList(allowMultiSelection=True)
        select_all_button = cmds.button(width=100, label='Select All',
                                        command=self.select_all_button_pressed)
        select_invert_button = cmds.button(width=100, label='Invert Selection',
                                           command=self.select_invert_button_pressed)
        select_none_button = cmds.button(width=100, label='Select None',
                                         command=self.select_none_button_pressed)
        update_list_button = cmds.button(width=100, label='Update List',
                                         command=self.refresh_bitmap_list)
        self.status_label = cmds.text(label='INFO: Standing by...', align='left')

        cmds.formLayout(main_layout, edit=True,
                        attachForm=[(tab_layout, 'top', 5),
                                    (tab_layout, 'left', 5),
                                    (self.bitmap_list_text, 'top', 5),
                                    (self.bitmap_list_text, 'right', 5),
                                    (self.bitmap_list, 'right', 5),
                                    (update_list_button, 'right', 5),
                                    (self.status_label, 'left', 5),
                                    (self.status_label, 'right', 5),
                                    (self.status_label, 'bottom', 5)],
                        attachControl=[(tab_layout, 'bottom', 5, self.status_label),
                                       (self.bitmap_list_text, 'left', 10, tab_layout),
                                       (select_all_button, 'left', 10, tab_layout),
                                       (select_all_button, 'bottom', 5, self.status_label),
                                       (select_invert_button, 'left', 10, select_all_button),
                                       (select_invert_button, 'bottom', 5, self.status_label),
                                       (select_none_button, 'left', 10, select_invert_button),
                                       (select_none_button, 'bottom', 5, self.status_label),
                                       (update_list_button, 'bottom', 5, self.status_label),
                                       (self.bitmap_list, 'top', 5, self.bitmap_list_text),
                                       (self.bitmap_list, 'left', 10, tab_layout),
                                       (self.bitmap_list, 'bottom', 5, select_all_button)])

        cmds.setParent('..')  # end formLayout

        self.refresh_bitmap_list()
        cmds.showWindow(bitmap_window)


def main():
    """Main entry point. Opens the UI"""
    bc = BitmapCleaner()
    bc.setup_ui()


if __name__ == "__main__":
    main()
